package com.kdm.erraiexample.client.marshaller;

import org.jboss.errai.enterprise.client.jaxrs.JacksonTransformer;
import org.jboss.errai.marshalling.client.api.json.EJArray;
import org.jboss.errai.marshalling.client.api.json.EJObject;
import org.jboss.errai.marshalling.client.api.json.EJValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gwt.json.client.JSONArray;
import com.google.gwt.json.client.JSONParser;
import com.google.gwt.json.client.JSONValue;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.Polygon;
/**
 * This class deserializes geojson into JTS Geometry objects.  It has been derived 
 * from the Jackson GeoJSON project:{"^ObjectID":"10", "^Value":[{"^ObjectID":"11", "^Value":[{"^ObjectID":"12", "^Value":[{"^ObjectID":"13", "^NumVal":-117.76469603701167},{"^ObjectID":"14", "^NumVal":57.692329503799144}]},{"^ObjectID":"15", "^Value":[{"^ObjectID":"16", "^NumVal":-117.76469603701167},{"^ObjectID":"17", "^NumVal":55.14112833650245}]},{"^ObjectID":"18", "^Value":[{"^ObjectID":"19", "^NumVal":-121.58793822451068},{"^ObjectID":"20", "^NumVal":55.14112833650245}]},{"^ObjectID":"21", "^Value":[{"^ObjectID":"22", "^NumVal":-121.58793822451068},{"^ObjectID":"23", "^NumVal":57.692329503799144}]},{"^ObjectID":"24", "^Value":[{"^ObjectID":"25", "^NumVal":-117.76469603701167},{"^ObjectID":"26", "^NumVal":57.692329503799144}]}]}]}
 *  
 * https://github.com/bedatadriven/jackson-geojson/blob/master/src/main/java/com/bedatadriven/geojson/jackson2/GeometryDeserializer.java
 */
public class GeometryDeserializer
{
  private final GeometryFactory gf = new GeometryFactory();
  private static final Logger log = LoggerFactory.getLogger(GeometryDeserializer.class);
  public Geometry deserialize(EJValue o)
  {
    EJObject obj = o.isObject();
    String typeName = obj.get("type").isString().stringValue();
    
    // TODO: handle other Geometry types; for now we only need Polygon...
    if (typeName.equals("Point"))
    {
      return gf.createPoint(parseCoordinate(obj.get("coordinates").isArray()));
    }
    else if (typeName.equals("Polygon")) 
    {
      EJValue ejValue = obj.get("coordinates");
      
      log.info("obj " + ejValue.isObject() + " arr " + ejValue.isArray() + " ejValue raw " + ejValue.getRawValue());
      
      String jsonString = JacksonTransformer.toJackson(ejValue.getRawValue().toString());
      JSONValue value = JSONParser.parseStrict(jsonString);
      JSONArray array = value.isArray();
      
      return parsePolygonCoordinates(array);
    }
    else
    {
      throw new UnsupportedOperationException("Unsupported geometry type: " + typeName);
    }
  }


  private Coordinate parseCoordinate(EJArray coords)
  {
    return new Coordinate(
        coords.get(0).isNumber().doubleValue(),
        coords.get(1).isNumber().doubleValue());
  }
  
  
  private Polygon parsePolygonCoordinates(JSONArray array)
  {
    log.info("-parsePolygonCoordinates:-----------------Array:"+array.toString());
    return gf.createPolygon(
        parseExteriorRing(array),
        parseInteriorRings(array));
  }
  
  private LinearRing parseExteriorRing(JSONArray array)
  {
    log.info("-parseExteriorRing-----------------Array:"+array.toString());
    JSONValue jsonValue = array.get(0);
    return gf.createLinearRing(parseLineString(jsonValue.isArray()));
  }

  private LinearRing[] parseInteriorRings(JSONArray array)
  {
    LinearRing rings[] = new LinearRing[array.size() - 1];
    for (int i = 1; i < array.size(); ++i) 
    {
      rings[i - 1] = gf.createLinearRing(parseLineString(array.get(i).isArray()));
    }
    return rings;  
  }
  private Coordinate[] parseLineString(JSONArray array)
  {
    Coordinate[] points = new Coordinate[array.size()];
    for (int i = 0; i != array.size(); ++i) 
    {
      points[i] = parseCoordinate(array.get(i).isArray());
    }
    return points;
  }  
  private Coordinate parseCoordinate(JSONArray coords)
  {
    return new Coordinate(
        coords.get(0).isNumber().doubleValue(),
        coords.get(1).isNumber().doubleValue());
  }
}
